@@ -1,12 +1,9 @@
-# -*- coding: utf-8 -*-
 """
 This module defines the main class that holds a DocTest example
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import __future__
 from collections import OrderedDict
 import traceback
-import six
 import warnings
 import math
 import sys
@@ -22,8 +19,13 @@ from xdoctest import global_state
 
 __devnotes__ = """
 TODO:
-    - [ ] Rename DocTest to Doctest?
-    - [ ] I dont like having "example" as a suffix to this modname, can we rename?
+    - [ ] Rename DocTest to Doctest? - Probably not, its been years.
+    - [ ] I dont like having "example" as a suffix to this modname, can we rename? - Probably not, its been years.
+"""
+
+
+__docstubs__ = """
+from xdoctest.doctest_part import DoctestPart
 """
 
 
@@ -201,6 +203,36 @@ class DocTest(object):
             how results are presented and what doctests are skipped.
             Can be "native" or "pytest". Defaults to "pytest".
 
+        config (DoctestConfig):
+            configuration for running / checking the doctest
+
+        module (ModuleType | None):
+            a reference to the module that contains the doctest
+
+        modname (str):
+            name of the module that contains the doctest.
+
+        failed_tb_lineno (int | None):
+            Line number a failure occurred on.
+
+        exc_info (None | TracebackType):
+            traceback of a failure if one occurred.
+
+        failed_part (None | DoctestPart):
+            the part containing the failure if one occurred.
+
+        warn_list (list):
+            from :func:`warnings.catch_warnings`
+
+        logged_evals (OrderedDict):
+            Mapping from part index to what they evaluated to (if anything)
+
+        logged_stdout (OrderedDict):
+            Mapping from part index to captured stdout.
+
+        global_namespace (dict):
+            globals visible to the doctest
+
     CommandLine:
         xdoctest -m xdoctest.doctest_example DocTest
 
@@ -539,7 +571,7 @@ class DocTest(object):
                         lineno=self.lineno, fpath=self.fpath)
             self._parts = parser.DoctestParser().parse(self.docsrc, info)
             self._parts = [p for p in self._parts
-                           if not isinstance(p, six.string_types)]
+                           if not isinstance(p, str)]
         # Ensure part numbers are given
         for partno, part in enumerate(self._parts):
             part.partno = partno
@@ -582,6 +614,9 @@ class DocTest(object):
                         # https://www.python.org/dev/peps/pep-3134/#explicit-exception-chaining
                         new_exc.__cause__ = None
                         raise new_exc
+                else:
+                    if global_state.DEBUG_DOCTEST:
+                        print('Pre import success: self.module={}'.format(self.module))
 
     @staticmethod
     def _extract_future_flags(namespace):
@@ -758,6 +793,10 @@ class DocTest(object):
                             return summary
 
                     test_globals, compileflags = self._test_globals()
+
+                    if  DEBUG:
+                        print('Global names = {}'.format(sorted(test_globals.keys())))
+
                     global_exec = self.config.getvalue('global_exec')
                     if global_exec:
                         # Hack to make it easier to specify multi-line input on the CLI
@@ -805,7 +844,6 @@ class DocTest(object):
                             # can compared to a "want" statement.
                             # print('part.compile_mode = {!r}'.format(part.compile_mode))
                             if part.compile_mode == 'eval':
-                                # print('test_globals = {}'.format(sorted(test_globals.keys())))
                                 got_eval = eval(code, test_globals)
                             else:
                                 exec(code, test_globals)
@@ -1291,7 +1329,7 @@ class DocTest(object):
     def _print_captured(self):
         out_text = ''.join([v for v in self.logged_stdout.values() if v])
         if out_text is not None:
-            assert isinstance(out_text, six.text_type), 'do not use ascii'
+            assert isinstance(out_text, str), 'do not use bytes'
         try:
             print(out_text)
         except UnicodeEncodeError:
