@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
 """
 The core logic that allows for xdoctest to parse source statically
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import sys
 from os.path import exists
 from os.path import isfile
@@ -11,8 +9,6 @@ from os.path import splitext
 import os
 import ast
 import re
-import six
-import tokenize
 from collections import deque, OrderedDict
 from xdoctest import utils
 
@@ -25,18 +21,43 @@ import platform
 PLAT_IMPL = platform.python_implementation()
 
 
-HAS_UPDATED_LINENOS = sys.version_info[0] >= 3 and sys.version_info[1] >= 8
+IS_PY_GE_308 = sys.version_info[0] >= 3 and sys.version_info[1] >= 8
+IS_PY_GE_312 = sys.version_info[0] >= 3 and sys.version_info[1] >= 12
+
+if IS_PY_GE_312:
+    from xdoctest import _tokenize as tokenize
+else:
+    import tokenize
 
 
 class CallDefNode(object):
     """
     Attributes:
-        callname (str): the name of the "calldef"
-        doclineno (int): the line number (1 based) the docstring begins on
-        doclineno_end (int): the line number (1 based) the docstring ends on
+        lineno_end (None | int):
+            the line number the docstring ends on (if known)
     """
     def __init__(self, callname, lineno, docstr, doclineno, doclineno_end,
                  args=None):
+        """
+        Args:
+            callname (str):
+                the name of the item containing the docstring.
+
+            lineno (int):
+                the line number the item containing the docstring.
+
+            docstr (str):
+                the docstring itself
+
+            doclineno (int):
+                the line number (1 based) the docstring begins on
+
+            doclineno_end (int):
+                the line number (1 based) the docstring ends on
+
+            args (None | ast.arguments):
+                arguments from static analysis :class:`TopLevelVisitor`.
+        """
         self.callname = callname
         self.lineno = lineno
         self.docstr = docstr
@@ -46,6 +67,10 @@ class CallDefNode(object):
         self.args = args
 
     def __str__(self):
+        """
+        Returns:
+            str
+        """
         return '{}[{}:{}][{}]'.format(
             self.callname, self.lineno, self.lineno_end,
             self.doclineno)
@@ -55,14 +80,20 @@ class TopLevelVisitor(ast.NodeVisitor):
     """
     Parses top-level function names and docstrings
 
-    For other visit_<classname> values see [MeetTheNodes]_.
+    For other ``visit_<classname>`` values see [MeetTheNodes]_.
 
     References:
-        ..[MeetTheNodes] http://greentreesnakes.readthedocs.io/en/latest/nodes.html
+        .. [MeetTheNodes] http://greentreesnakes.readthedocs.io/en/latest/nodes.html
 
     CommandLine:
         python -m xdoctest.static_analysis TopLevelVisitor
 
+    Attributes:
+        calldefs (OrderedDict):
+        source (None | str):
+        sourcelines (None | List[str]):
+        assignments (list):
+
     Example:
         >>> from xdoctest.static_analysis import *  # NOQA
         >>> from xdoctest import utils
@@ -264,13 +295,22 @@ class TopLevelVisitor(ast.NodeVisitor):
         """
         if isinstance(node.test, ast.Compare):  # pragma: nobranch
             try:
-                if all([
-                    isinstance(node.test.ops[0], ast.Eq),
-                    node.test.left.id == '__name__',
-                    node.test.comparators[0].s == '__main__',
-                ]):
-                    # Ignore main block
-                    return
+                if IS_PY_GE_312:
+                    if all([
+                        isinstance(node.test.ops[0], ast.Eq),
+                        node.test.left.id == '__name__',
+                        node.test.comparators[0].value == '__main__',
+                    ]):
+                        # Ignore main block
+                        return
+                else:
+                    if all([
+                        isinstance(node.test.ops[0], ast.Eq),
+                        node.test.left.id == '__name__',
+                        node.test.comparators[0].s == '__main__',
+                    ]):
+                        # Ignore main block
+                        return
             except Exception:  # nocover
                 pass
         self.generic_visit(node)  # nocover
@@ -358,7 +398,10 @@ class TopLevelVisitor(ast.NodeVisitor):
         else:
             if PLAT_IMPL == 'PyPy':
                 startpos = docnode.lineno - 1
-                docstr = utils.ensure_unicode(docnode.value.s)
+                if IS_PY_GE_312:
+                    docstr = utils.ensure_unicode(docnode.value.value)
+                else:
+                    docstr = utils.ensure_unicode(docnode.value.s)
                 sourcelines = self.sourcelines
                 start, stop = self._find_docstr_endpos_workaround(docstr,
                                                                   sourcelines,
@@ -372,7 +415,10 @@ class TopLevelVisitor(ast.NodeVisitor):
                 # TODO: fix in pypy
                 endpos = docnode.lineno - 1
 
-        docstr = utils.ensure_unicode(docnode.value.s)
+        if IS_PY_GE_312:
+            docstr = utils.ensure_unicode(docnode.value.value)
+        else:
+            docstr = utils.ensure_unicode(docnode.value.s)
         sourcelines = self.sourcelines
         start, stop = self._find_docstr_startpos_workaround(docstr, sourcelines, endpos)
         # Convert 0-based line positions to 1-based line numbers
@@ -524,7 +570,10 @@ class TopLevelVisitor(ast.NodeVisitor):
             >>>     print('docnode = {!r}'.format(docnode))
             >>>     print('docnode.value = {!r}'.format(docnode.value))
             >>>     print('docnode.value.__dict__ = {!r}'.format(docnode.value.__dict__))
-            >>>     print('docnode.value.s = {!r}'.format(docnode.value.s))
+            >>>     if IS_PY_GE_312:
+            >>>         print('docnode.value.value = {!r}'.format(docnode.value.value))
+            >>>     else:
+            >>>         print('docnode.value.s = {!r}'.format(docnode.value.s))
             >>>     print('docnode.lineno = {!r}'.format(docnode.lineno))
             >>>     print('docnode.col_offset = {!r}'.format(docnode.col_offset))
             >>>     print('docnode = {!r}'.format(docnode))
@@ -671,9 +720,6 @@ def parse_static_calldefs(source=None, fpath=None):
         >>> calldefs = parse_static_calldefs(fpath=fpath)
         >>> assert 'parse_static_calldefs' in calldefs
     """
-    if six.PY2:
-        fpath = fpath.replace('.pyc', '.py')
-
     if source is None:  # pragma: no branch
         try:
             with open(fpath, 'rb') as file_:
@@ -714,8 +760,8 @@ def _parse_static_node_value(node):
     # TODO: ast.Constant for 3.8
     if isinstance(node, ast.Num):
         value = node.n
-    elif isinstance(node, ast.Str):
-        value = node.s
+    elif (isinstance(node, ast.Constant) and isinstance(node.value, str) if IS_PY_GE_308 else isinstance(node, ast.Str)):
+        value = node.value if IS_PY_GE_308 else node.s
     elif isinstance(node, ast.List):
         value = list(map(_parse_static_node_value, node.elts))
     elif isinstance(node, ast.Tuple):
@@ -725,12 +771,8 @@ def _parse_static_node_value(node):
         values = map(_parse_static_node_value, node.values)
         value = OrderedDict(zip(keys, values))
         # value = dict(zip(keys, values))
-    elif six.PY3 and isinstance(node, (ast.NameConstant)):
+    elif isinstance(node, (ast.NameConstant)):
         value = node.value
-    elif (six.PY2 and isinstance(node, ast.Name) and
-          node.id in ['None', 'True', 'False']):
-        # disregard pathological python2 corner cases
-        value = {'None': None, 'True': True, 'False': False}[node.id]
     else:
         print(node.__dict__)
         raise TypeError('Cannot parse a static value from non-static node '
@@ -876,12 +918,13 @@ def is_balanced_statement(lines, only_tokens=False, reraise=0):
         bool: True if the statement is balanced, otherwise False
 
     CommandLine:
-        xdoctest -m xdoctest.static_analysis is_balanced_statement
+        xdoctest -m xdoctest.static_analysis is_balanced_statement:0
 
     References:
         https://stackoverflow.com/questions/46061949/parse-until-complete
 
-    Doctest:
+    Example:
+        >>> from xdoctest.static_analysis import *  # NOQA
         >>> assert is_balanced_statement(['print(foobar)'])
         >>> assert is_balanced_statement(['foo = bar']) is True
         >>> assert is_balanced_statement(['foo = (']) is False
@@ -895,7 +938,7 @@ def is_balanced_statement(lines, only_tokens=False, reraise=0):
         >>> lines = ['def foo():', '', '    x = 1', 'assert True', '']
         >>> assert is_balanced_statement(lines)
 
-    Doctest:
+    Example:
         >>> from xdoctest.static_analysis import *
         >>> source_parts = [
         >>>     'setup(',
@@ -925,12 +968,19 @@ def is_balanced_statement(lines, only_tokens=False, reraise=0):
         >>> print('\n'.join(source_parts))
         >>> assert is_balanced_statement(source_parts)
 
-    Doctest:
+    Example:
         >>> lines = ['try: raise Exception']
         >>> is_balanced_statement(lines, only_tokens=1)
         True
         >>> is_balanced_statement(lines, only_tokens=0)
         False
+
+    Example:
+        >>> # Cause a failure case on 3.12
+        >>> from xdoctest.static_analysis import *
+        >>> lines = ['3, 4]', 'print(len(x))']
+        >>> is_balanced_statement(lines, only_tokens=1)
+        False
     """
     # Only iterate through non-empty lines otherwise tokenize will stop short
     lines = list(lines)
@@ -942,7 +992,8 @@ def is_balanced_statement(lines, only_tokens=False, reraise=0):
             pass
     except tokenize.TokenError as ex:
         message = ex.args[0]
-        if message.startswith('EOF in multi-line'):
+        # First case is Python <= 3.11, Second case is >= 3.12
+        if message.startswith(('EOF in multi-line', 'unexpected EOF in multi-line')):
             if reraise:
                 raise
             return False
@@ -997,7 +1048,7 @@ def extract_comments(source):
         >>> comments = list(extract_comments(source.splitlines()))
         >>> assert comments == ['# comment 1', '# comment 2']
     """
-    if isinstance(source, six.string_types):
+    if isinstance(source, str):
         lines = source.splitlines()
     else:
         lines = source
@@ -1055,7 +1106,7 @@ def _strip_hashtag_comments_and_newlines(source):
         >>> assert non_comments.count(chr(10)) == 10
         >>> assert non_comments.count('#') == 1
     """
-    if isinstance(source, six.string_types):
+    if isinstance(source, str):
         import io
         f = io.StringIO(source)
         readline = f.readline
@@ -1109,17 +1160,7 @@ def six_axt_parse(source_block, filename='<source_block>', compatible=True):
     Returns:
         ast.Module | types.CodeType
     """
-    # Note Python2.7 does not accept unicode variable names so this
-    # will fail (in 2.7) if source contains a unicode varname.
-    if compatible and six.PY2:
-        # In Python2.7 fix __future__ issues
-        import __future__
-        flags = ast.PyCF_ONLY_AST
-        flags |= __future__.print_function.compiler_flag
-        # flags |= __future__.print_function.unicode_literals
-        pt = compile(source_block, filename=filename, mode='exec', flags=flags)
-    else:
-        pt = ast.parse(source_block, filename=filename)
+    pt = ast.parse(source_block, filename=filename)
     return pt
 
 
